xquery version "3.1";

module namespace contentapi="http://sepia.io/content-parser/content-api";

import module namespace app="https://sade.textgrid.de/ns/app" at "../app.xqm";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";

declare namespace http="http://expath.org/ns/http-client";
declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";

declare variable $contentapi:repo-id := config:get("contentapi.repo", "contentapi");
declare variable $contentapi:secret := config:get("contentapi.secret", "contentapi");
declare variable $contentapi:target := config:get("contentapi.target", "contentapi");

declare function contentapi:content($node as node(), $model as map(*), $id as xs:string)
{
    let $configLang := string(config:get("lang.default", "multilanguage"))
    let $lang := if($configLang = "") then "" else ("_" || app:getLanguage())
    let $requestUrl := "https://gitlab.gwdg.de/api/v4/projects/" || $contentapi:repo-id || "/repository/files/" || $id || $lang || ".md?ref=main"
    let $version := contentapi:check-version($id, $lang)
    let $document :=
        if ($version != "none") then
            doc($contentapi:target || "/" || $id || $lang || '.hash' || $version || '.html' )
        else
            contentapi:update-document($id, $lang)

    return
        $document
};

declare function contentapi:update-document($id, $lang)
{
    let $markdownRequest := <http:request method='get' ><http:header name="PRIVATE-TOKEN" value='{$contentapi:secret}' /></http:request>
    let $markdownRequestUrl := "https://gitlab.gwdg.de/api/v4/projects/" || $contentapi:repo-id || "/repository/files/" || $id || $lang || ".md?ref=main"
    let $rest := http:send-request($markdownRequest, $markdownRequestUrl)
    let $version := $rest[1]//*[@name="x-gitlab-last-commit-id"]/@value
    let $markdown := http:send-request($markdownRequest, $markdownRequestUrl)[2]
    let $markdown := $markdown => util:base64-decode() => parse-json() => map:get("content") => util:base64-decode()

    let $transformRequest := <http:request method="post"><http:header name="PRIVATE-TOKEN" value='S9iaMUmSh_bnTT2R7zTj' /><http:body media-type="application/json" method="text"/></http:request>
    let $transformRequestUrl := "https://gitlab.gwdg.de/api/v4/markdown"
    let $payload := serialize(map{"text": $markdown}, <output:serialization-parameters><output:method>json</output:method></output:serialization-parameters>)
    let $responseBody := http:send-request($transformRequest, $transformRequestUrl, $payload)[2]

    let $content := $responseBody => util:base64-decode() => parse-json() => map:get("html") => util:parse-html()
    let $login := xmldb:login("/db", "admin", "" )
    let $save := if ($content) then xmldb:store($contentapi:target, $id || $lang || ".hash" || $version || ".html", $content) else ()

    return
        $content
};

declare function contentapi:check-version($id as xs:string, $lang as xs:string)
{
    let $local := contentapi:get-current-version($id, $lang)
    let $remote := contentapi:get-remote-version($id, $lang)

    return
        if ($local = $remote and $local != "none") then
            $local
        else
            "none"
};

declare function contentapi:get-current-version($id as xs:string, $lang as xs:string)
{
    let $version := xmldb:get-child-resources($contentapi:target)[contains(., $id) and contains(., $lang)][1]
        => substring-after(".hash")
        => substring-before(".html")

    return
        if ($version) then
            $version
        else
            "none"
};

declare function contentapi:get-remote-version($id as xs:string, $lang as xs:string)
{
    let $httpRequest := <http:request method='get' ><http:header name="PRIVATE-TOKEN" value='{$contentapi:secret}' /></http:request>
    let $requestUrl := "https://gitlab.gwdg.de/api/v4/projects/" || $contentapi:repo-id || "/repository/files/" || $id || $lang || ".md?ref=main"
    let $rest := http:send-request($httpRequest, $requestUrl)

    return
        if ($rest[1]/@status = 200) then
            $rest[1]//*[@name="x-gitlab-last-commit-id"]/@value
        else
            "none"
};
